import pylab
from matplotlib import pyplot
from PlotInfo import PlotInfo

class Line(PlotInfo):
    def __init__(self):
        super(Line,self).__init__("line")
        
        self.marker = None
        self.markerSize = 8.0
        # TODO Change to width
        self.lineWidth = 1
        self.color = 'black'
        self.lineStyle = '-'
        self.dates = False
        self.loglog = False
        self.steps = None

    def stepFunction(self, stepType="pre"):
        validStepTypes = ["pre", "mid", "post"]
        
        if stepType not in validStepTypes:
            print >>sys.stderr, "%s is not a valid step type. Valid step types are %s" % (stepType, ", ".join(validStepTypes))
            sys.exit(1)
        
        self.steps = stepType

    def draw(self, axis):
        super(Line, self).draw(axis)
        
        if self.dates:
            plotFunc = axis.plot_date
        elif self.loglog:
            print >>sys.stderr, "Setting loglog in Lines will be deprecated soon. Set this in Plot instead."
            plotFunc = axis.loglog
        else:
            plotFunc = axis.plot

        kwdict = {}
        kwdict["linestyle"] = self.lineStyle
        kwdict["color"] = self.color
        kwdict["label"] = self.label
        kwdict["linewidth"] = self.lineWidth
        
        if self.steps is not None:
            kwdict["drawstyle"] = "steps-%s" % (self.steps)
        
        if self.marker is not None:
            kwdict["marker"] = self.marker
            kwdict["markersize"] = self.markerSize
        else:
            kwdict["marker"] = "None"
        
        return [[plotFunc(self.xValues, self.yValues, **kwdict)], [self.label]]



import pylab
from matplotlib import pyplot
import sys
from PlotInfo import PlotInfo
from PlotLayout import *
import copy

try:
    import mpl_toolkits.axes_grid.inset_locator
    insetLocatorLoaded = True
except ImportError:
    insetLocatorLoaded = False

from Utils import getGoldenRatioDimensions

class Plot(object):
    """
    Represents a single plot, usually consisting of a single X and Y axis.
    """

    def __init__(self):
        self.plots = []
        self.title = None
        self.xFormatter = None
        self.yFormatter = None
        self.yLabel = None
        self.xLabel = None
        self.legend = False

        self.xlim = None
        self.ylim = None

        self.legendCols = 0
        self.scatterPoints=3
        self.legendLoc = None

        self.twinxLabel = None
        self.twinxIndex = -1
        self.twinxLimits = None

        self.lineStyles = None
        self.lineColors = None
        self.markers = None

        self.width = None
        self.height = None
        
        self.plotParams = None
        self.logx = False
        self.logy = False
        self.loglog = False
        self.logbase = 10
        self.logbasex = None
        self.logbasey = None
        
        self.grid = False

        self.figLegend = False
        
        self.insets = []
        
        self.hideTicks = False
        
        self.latex = False
        
        self.axesLabelSize = None
        self.xTickLabelSize = None
        self.yTickLabelSize = None
        self.legendLabelSize = None
        
        self.titleProperties = {}

        self.tight = False
        
    def setTitleProperties(self, **propList):
        self.__setProperties(self.titleProperties, propList)

    def __setProperties(self, propsDict, propList):
        # Going to restrict the set of properties that can be modified so they
        # don't mess with the rest of the system
        
        validProperties = ["alpha", "backgroundColor", "color", 
                           "horizontalalignment", "linespacing", 
                           "multialignment", "rotation", "stretch", "style", 
                           "verticalalignment", "weight"]
        
        for (key, val) in propList.items():
            if key not in validProperties:
                print >>sys.stderr, "Property '%s' is not currently supported" % (key)
            else:
                propsDict[key] = val

    def __str__(self):
        return str(self.__dict__)
    
    def setAxesLabelSize(self, size):
        self.axesLabelSize = size
    
    def setXTickLabelSize(self, size):
        self.xTickLabelSize = size
    
    def setYTickLabelSize(self, size):
        self.yTickLabelSize = size
    
    def setLegendLabelSize(self, size):
        self.legendLabelSize = size
    
    def split(self, pieces):
       splitPlots = [copy.deepcopy(self) for i in xrange(pieces)]
       
       for plot in splitPlots:
           plot.plots = []
       
       for plot in self.plots:
           elements = plot.split(pieces)
           for i in xrange(pieces):
               splitPlots[i].add(elements[i])
               splitPlots[i].setXLimits(min(elements[i].xValues), max(elements[i].xValues))
       return splitPlots
       
    def getDimensions(self):
        if self.width is None:
            (self.width, self.height) = getGoldenRatioDimensions(8.0)
        elif self.height is None:
            (goldWidth, goldHeight) = getGoldenRatioDimensions(self.width)
            self.height = goldHeight
        return (self.width, self.height)

    def hideTickLabels(self):
        self.hideTicks = True

    def setDimensions(self, width=None, height=None):
        self.width = width
        self.height = height

    def add(self, plottableObject):
        """
        Add a plottable object (a Line, Bar, etc.) to this plot, causing it to
        be drawn when the plot is drawn.
        """

        if not issubclass(plottableObject.__class__, PlotInfo):
            print >>sys.stderr, "All objects added to a Plot must be a subclass of PlotInfo"
            sys.exit(1)
        self.plots.append(plottableObject)

    def addInset(self, inset, width=0.3, height=0.3, location="upper right", 
                 padding=0.05):
        if not isinstance(inset, Plot):
            print >>sys.stderr, "Can only add Plots as insets"
            sys.exit(1)

        if not isinstance(width, float) or not isinstance(height, float):
            print >>sys.stderr, \
                "Width and height of inset must be numbers in range [0.0, 1.0)"
            sys.exit(1)
        
        self.insets.append((inset, width, height, location, padding))

    def addLineStyle(self, style):
        """
        Add a line style to the list of line styles through which the plot will
        cycle when drawing lines. If no line styles are specified, the plot
        will default to the line style specified in the Line objects
        themselves.
        
        Note that, when drawing lines, all line styles for a given color are
        cycled through before another color is used.
        """
        if self.lineStyles is None:
            self.lineStyles = []
        self.lineStyles.append(style)

    def addLineColor(self, color):
        """
        Add a line color to the list of line colors through which the plot will
        cycle when drawing lines. If no line colors are specified, the line
        colors specified by the Line objects themselves will be used.
        
        Note that, when drawing lines, all line styles for a given color are
        cycled through before another color is used.
        """
        if self.lineColors is None:
            self.lineColors = []
        self.lineColors.append(color)

    def addMarker(self, marker):
        if self.markers is None:
            self.markers = []
        self.markers.append(marker)

    def getNumPlots(self):
        """
        Get the number of plottable objects currently registered for this plot.
        """
        return len(self.plots)
    
    def setTwinX(self, label, index, yMin=None, yMax=None):
        """
        Make the plot use a secondary y-axis with the provided label. All
        registered plottable objects at or after the given index will use this
        second axis.
        """
        self.twinxLabel = label
        self.twinxIndex = index

        if yMin == None and yMax == None:
            self.twinxLimits = (yMin, yMax)

    def setYFormatter(self, formatter):
        """
        Set the y-axis formatter used by this plot to the given function.
        """
        self.yFormatter = pylab.FuncFormatter(formatter)

    def setXFormatter(self, formatter):
        """
        Set the x-axis formatter used by this plot to the given function.
        """
        self.xFormatter = pylab.FuncFormatter(formatter)

    def setXLabel(self, xLabel):
        """
        Set the x-axis label.
        """
        self.xLabel = xLabel

    def setYLabel(self, yLabel):
        """
        Set the y-axis label.
        """
        self.yLabel = yLabel

    def setXLimits(self, minX, maxX):
        """
        Set the minimum and maximum values for the x-axis.
        """
        self.xlim = (minX, maxX)

    def setYLimits(self, minY, maxY):
        """
        Set the minimum and maximum values for the y-axis.
        """
        self.ylim = (minY, maxY)

    def hasFigLegend(self, columns=1, location="best", scatterPoints=3, 
                     draw_frame=True):
        self.figLegend = True
        self.legendCols = columns
        self.legendLoc = location
        self.scatterPoints = scatterPoints
        self.legendDrawFrame = draw_frame

    def hasLegend(self, columns=1, location="best", scatterPoints=3,
                  draw_frame=True):
        """
        Declare that the plot has a legend with a given number of columns and
        location.
        """
        self.legend = True
        self.legendCols = columns
        self.legendLoc = location
        self.scatterPoints = scatterPoints
        self.legendDrawFrame = draw_frame

    def setTitle(self, title):
        self.title = title

    def __cmp__(self, other):
        assert(isinstance(other, Plot))

        return cmp(self.title, other.title)

    def __setupLayout(self):
        layout = PlotLayout()
        (width, height) = self.getDimensions()
        
        layout.setPlotDimensions(width, height)

        if self.plotParams is not None:
            layout.setPlotParameters(**self.plotParams)

        if self.latex == True:
            layout.useLatexLabels()

        if self.axesLabelSize is not None:
            layout.setAxesLabelSize(self.axesLabelSize)
        
        if self.xTickLabelSize is not None:
            layout.setXTickLabelSize(self.xTickLabelSize)
        
        if self.yTickLabelSize is not None:
            layout.setYTickLabelSize(self.yTickLabelSize)
                
        layout.addPlot(self)
        return layout

    def useLatexLabels(self):
        self.latex = True

    def plot(self):
        """
        Draw this plot to a matplotlib canvas.
        """
        layout = self.__setupLayout()
        layout.plot()

    def setPlotParameters(self, **kwdict):
        self.plotParams = dict(kwdict)

        if "left" not in self.plotParams:
            self.plotParams["left"] = 0.12

        if "bottom" not in self.plotParams:
            self.plotParams["bottom"] = 0.10

        if "right" not in self.plotParams:
            self.plotParams["right"] = 0.90

        if "top" not in self.plotParams:
            self.plotParams["top"] = 0.90

        if "wspace" not in self.plotParams:
            self.plotParams["wspace"] = 0.20

        if "hspace" not in self.plotParams:
            self.plotParams["hspace"] = 0.20


    def save(self, filename, **kwargs):
        """
        Save this plot to a file.
        """
        layout = self.__setupLayout()
        layout.save(filename,**kwargs)

    def plotInset(self, parentAxes, width, height, location, padding):
        if not insetLocatorLoaded:
            print sys.stderr, "Plotting insets requires mpl_toolkits.axes_grid.inset_locatoor, which your version of matplotlib doesn't appear to have."
            sys.exit(1)

        locationMap = {"best" : 0, 
                       "upper right" : 1,
                       "upper left" : 2, 
                       "lower left" : 3,
                       "lower right" : 4, 
                       "right" : 5, 
                       "center left" : 6, 
                       "center right" : 7,
                       "lower center" : 8, 
                       "upper center" : 9, 
                       "center" : 10}

        if location not in locationMap:
            print >>sys.stderr, "Location '%s' isn't valid. Valid locations are: %s" % (location, ', '.join(locationMap))

        ax = mpl_toolkits.axes_grid.inset_locator.inset_axes(parentAxes, width="%.2f%%" % (width * 100.0), height="%.2f%%" % (height * 100.0), loc=locationMap[location])
        return self.drawPlot(ax)

    # def plotInset(self, parentAxes, width, height, hPos, vPos, padding):
    #     print parentAxes
    #     insetWidth = width
    #     insetHeight = height
        
    #     parentBBox = parentAxes.get_position().get_points()

    #     (parentLeft, parentBottom) = parentBBox[0]
    #     (parentWidth, parentHeight) = parentBBox[1]

    #     insetLeft = parentLeft
    #     insetBottom = parentBottom
    #     width = width * parentWidth
    #     height = height * parentHeight

    #     if vPos == "lower":
    #         insetBottom += padding
    #     elif vPos == "upper":
    #         insetBottom += parentHeight - (padding + height)

    #     if hPos == "left":
    #         insetLeft += padding
    #     elif hPos == "center":
    #         insetLeft += parentWidth - (width / 2.0)
    #     elif hPos == "right":
    #         insetLeft += parentWidth - (padding + width)
        
    #     print insetLeft, insetBottom, width, height
        
    #     ax = pylab.axes([insetLeft, insetBottom, width, height])
    #     return self.drawPlot(ax)

    def subplot(self, row, column, position):
        ax = pylab.subplot(row, column, position)
        return self.drawPlot(ax)
        
    def drawPlot(self, ax):
        """
        Used by PlotLayout to plot the graph at a given location in the layout.
        """

        if self.tight:
            ax.autoscale_view(tight=True)
            
        for insetInfo in self.insets:
            insetInfo[0].plotInset(ax, *(insetInfo[1:]))
        
        if self.hideTicks == True:
            for xtl in ax.get_xticklabels():
                xtl.set_visible(False)
            for xtick in ax.get_xticklines():
                xtick.set_visible(False)
            for ytl in ax.get_yticklabels():
                ytl.set_visible(False)
            for ytick in ax.get_yticklines():
                ytick.set_visible(False)

        
        if self.grid:
            #TODO: color and linestyle for gridlines should be configurable
            ax.grid(color="#dddddd", linestyle="-")
            # Gridlines should be below plots
            ax.set_axisbelow(True)

        if self.loglog or self.logx:
            myBase = None
            
            if self.logbasex is not None:
                myBase = self.logbasex
            else:
                myBase = self.logbase
            
            ax.set_xscale('log', basex=myBase)
        if self.loglog or self.logy:
            myBase = None
            
            if self.logbasey is not None:
                myBase = self.logbasey
            else:
                myBase = self.logbase
            
            ax.set_yscale('log', basey=myBase)

        if self.twinxIndex > 0:
            ax2 = ax.twinx()
            ax2.set_ylabel(self.twinxLabel)
            if self.twinxLimits is not None:
                ax2.set_ylim(ymin=self.twinxLimits[0], ymax=self.twinxLimits[1])

        plotHandles = []
        plotLabels = []

        if self.xFormatter is not None:
            ax.xaxis.set_major_formatter(self.xFormatter)

        if self.yFormatter is not None:
            ax.yaxis.set_major_formatter(self.yFormatter)

        if self.title is not None:
            ax.set_title(self.title, **self.titleProperties)

        i = 0
        myAxis = ax
        
        hasLineStyles = self.lineStyles is not None
        hasColors = self.lineColors is not None
        hasMarkers = self.markers is not None
        
        numLineStyles = 1
        numColors = 1
        numMarkers = 1
                
        if hasLineStyles:
            numLineStyles = len(self.lineStyles)
            
        if hasColors:
            numColors = len(self.lineColors)

        if hasMarkers:
            numMarkers = len(self.markers)

        plotIndex = 0
        
        for plotInfo in self.plots:
            if self.twinxIndex >= 0 and i == self.twinxIndex:
                myAxis = ax2
                
            myLineStyle = None
            myColor = None
            myMarker = None
            
            # cycle through styles first, then markers, then colors
            colorIndex = (plotIndex / (numMarkers * numLineStyles)) % numColors
            markerIndex = (plotIndex / numLineStyles) % numMarkers
            lineStyleIndex = plotIndex % numLineStyles

            if hasLineStyles:
                myLineStyle = self.lineStyles[lineStyleIndex]
            
            if hasColors:
                myColor =  self.lineColors[colorIndex]
            
            if hasMarkers:
                myMarker = self.markers[markerIndex]

                                
            plotIndex += 1
                
            if myLineStyle is not None:
                plotInfo.lineStyle = myLineStyle

            if myMarker is not None:
                plotInfo.marker = myMarker
            
            if myColor is not None:
                plotInfo.color = myColor
            
            (currPlotHandles, currPlotLabels) = plotInfo.draw(myAxis)
            
            labelIndices = [x for x in range(len(currPlotLabels)) \
                                if currPlotLabels[x] is not None]
            
            if len(labelIndices) > 0:
                plotHandles.extend([currPlotHandles[x] for x in labelIndices])
                plotLabels.extend([currPlotLabels[x] for x in labelIndices])
            
            if plotInfo.xLimits is not None:
                if self.xlim is None:
                    self.xlim = plotInfo.xLimits
                else:
                    (myXMin, myXMax) = plotInfo.xLimits
                    self.xlim = (min(self.xlim[0], myXMin), 
                                 max(self.xlim[1], myXMax))
            
            i += 1
        
        if self.xlim is not None:
            pylab.xlim(xmin=self.xlim[0], xmax=self.xlim[1])
        
        if self.ylim is not None:
            pylab.ylim(ymin=self.ylim[0], ymax=self.ylim[1])

        
        if self.xLabel is not None:
            ax.set_xlabel(self.xLabel)

        if self.yLabel is not None:
            ax.set_ylabel(self.yLabel)

        legendKeywords = {}

        if self.legendCols > 0:
            versionParts = [int(x) for x in matplotlib.__version__.split('.')]
            
            (superMajor, major, minor) = versionParts[0:3]

            if superMajor == 0 and major < 98:
                print >>sys.stderr, "Number of columns support not available in versions of matplotlib prior to 0.98"
            else:
                legendKeywords["ncol"] = self.legendCols
                legendKeywords["scatterpoints"] = self.scatterPoints
                
                if self.legendLabelSize is not None:
                    legendKeywords["prop"] = {"size" : self.legendLabelSize}

        legend = None # So we can disable the box if we want

        if self.legend:
            if len(plotHandles) == 0:
                print >>sys.stderr, "ERROR: Plot wanted to draw a legend, but none of its elements have labels"
                sys.exit(1)
            legend = pylab.legend(plotHandles, plotLabels,
                                  loc=self.legendLoc, **legendKeywords)
        if self.figLegend:
            legend = pylab.figlegend(plotHandles, plotLabels,
                                     loc=self.legendLoc, **legendKeywords)

        if legend:
            legend.draw_frame(self.legendDrawFrame)
 
        return (plotHandles, plotLabels)



import math
import re
from Line import Line
from Bar import Bar

def getGoldenRatioDimensions(width):
    goldenRatio = (math.sqrt(5) - 1.0) / 2.0
    return (width, goldenRatio * width)

def getXYValsFromFile(filename, regex, postFunction=None, 
                      autofillXValues=False):
    fp = open(filename)
    
    regex = re.compile(regex)
    
    matches = []

    i = 0
    for line in fp:
        line = line.strip()
        
        match = regex.match(line)
        
        if match is not None:
            matchGroups = match.groups()
                        
            if postFunction is not None:
                matchGroups = postFunction(matchGroups)
            else:
                matchGroups = [float(x) for x in matchGroups]
            
            if len(matchGroups) < 2 or autofillXValues:
                matchGroups.insert(0, i)

            matches.append(matchGroups)
            i += 1
    fp.close()
    
    matches.sort()
    
    xValues = []
    yValues = []
    
    for matchGroups in matches:

        numMatchGroups = len(matchGroups)
        
        if len(yValues) == 0:
            yValues = [[] for i in xrange(numMatchGroups - 1)]
        
        xValues.append(matchGroups[0])
        for i in xrange(1, numMatchGroups):
            yValues[i-1].append(matchGroups[i])
        
    
    return [xValues, yValues]

def getLinesFromFile(filename, regex, postFunction=None, autofillXValues=False):
    (xValues, yValues) = getXYValsFromFile(filename, regex, postFunction, 
                                           autofillXValues)
    
    lines = []
    
    for i in xrange(len(yValues)):
        line = Line()
        line.xValues = xValues[:]
        line.yValues = yValues[i][:]
        lines.append(line)
    return lines
    
def getBarsFromFile(filename, regex, postFunction=None, autofillXValues=False):
    (xValues, yValues) = getXYValsFromFile(filename, regex, postFunction, 
                                           autofillXValues)
    
    bars = []
    
    for i in xrange(len(yValues)):
        bar = Bar()
        bar.xValues = xValues[:]
        bar.yValues = yValues[i][:]
        bars.append(bar)
    return bars

def getCDF(values):
    line = Line()
    cdfValues = values[:]
    cdfValues.sort()
    
    count = float(len(cdfValues))

    line.xValues = cdfValues
    line.yValues = [float(x) / count for x in xrange(1, int(count) + 1)]
    assert(count == len(line.yValues))
    
    return line



import pylab
import matplotlib
from matplotlib import pyplot
import sys
import os
from boomslang import PlotLayout

from Utils import getGoldenRatioDimensions

class WeightedPlotLayout(PlotLayout):
    def __init__(self):
        super(WeightedPlotLayout,self).__init__()
        self.groupedWeights = {}
        self.weights = []
        self.figTitle = None

    def addPlot(self, plot, grouping=None, weight=1):
        super(WeightedPlotLayout,self).addPlot(plot, grouping=grouping)

        if grouping == None:
            self.weights.append(weight)
        else:
            if grouping not in self.groupedWeights:
                self.groupedWeights[grouping] = []
            self.groupedWeights[grouping].append(weight)

    def setFigTitle(self, title=None):
        self.figTitle = title

    def _doPlot(self):
        if len(self.groupedPlots) + len(self.plots) == 0:
            print "WeightedPlotLayout.plot(): No data to plot!"
            return

        if self.rcParams is not None:
            pylab.rcParams.update(self.rcParams)

        numRows = len(self.groupedPlots.keys()) + len(self.plots)
        maxRowLength = max([len(self.groupedPlots[f])
                                for f in self.groupedPlots.keys()])

        if self.groupOrder is not None:
            keyList = self.groupOrder
        else:
            keyList = self.groupedPlots.keys()

        if self.figdimensions is not None:
            fig = pyplot.figure(figsize=(self.figdimensions[0],
                                         self.figdimensions[1]))
        elif self.dimensions is not None:
            fig = pyplot.figure(figsize=(self.dimensions[0] * maxRowLength,
                                         self.dimensions[1] * numRows))
        else:
            (figWidth, figHeight) = getGoldenRatioDimensions(8.0)
            figWidth *= maxRowLength
            figHeight *= numRows
            fig = pyplot.figure(figsize=(figWidth, figHeight))
            # figWidth = fig.get_figwidth()
            # print figWidth
            # print fig.get_figheight()
            # (goldenWidth, goldenHeight) = getGoldenRatioDimensions(figWidth)
            # fig.set_figheight(goldenHeight)
            # print fig.get_figheight()

        plotHandles = []
        plotLabels = []

        # Force a call to plotParams since we need them here
        if self.plotParams is None:
            self.setPlotParameters(**dict())

        figTop = self.plotParams["top"]
        figLeft = self.plotParams["left"]

        wspace = self.plotParams["wspace"]
        hspace = self.plotParams["hspace"]

        height = figTop - self.plotParams["bottom"]
        rowHeight = height / (numRows + (numRows - 1) * hspace)
        hgap = self.plotParams["hspace"] * rowHeight

        rowWidth = self.plotParams["right"] - figLeft

        # To contain a list of plots and rects, so we can do the
        # information collection in one pass
        plotInfo = []

        # Generate rects for grouped plots
        currentRow = 0
        for grouping in keyList:
            plots = self.groupedPlots[grouping]
            weights = self.groupedWeights[grouping]

            totalWeight = 1.0 * sum(weights)
            numPlots = len(plots)

            # hspace, wspace behavior defined in matplotlib/axes.py
            # in the class SubplotBase
            unitWidth = rowWidth / (numPlots + (numPlots-1) * wspace)
            availableWidth = unitWidth * numPlots
            wgap = unitWidth * wspace

            bottom = figTop - rowHeight - (rowHeight + hgap) * currentRow
            left = figLeft

            for i in range(0, len(plots)):
                plot = plots[i]
                weight = weights[i]
                myWidth = availableWidth * weights[i] / totalWeight

                plotInfo.append((plot, [left, bottom, myWidth, rowHeight]))

                left += myWidth + wgap

            currentRow += 1

        # Generate rects for ungrouped plots
        for plot in self.plots:
            bottom = figTop - rowHeight - (rowHeight + hgap) * currentRow
            left = figLeft
            plotInfo.append((plot, [left, bottom, rowWidth, rowHeight]))

            currentRow += 1

        # Plot everything
        for (plot, rect) in plotInfo:
            ax = fig.add_axes(rect)
            (currPlotHandles, currPlotLabels) = plot.drawPlot(ax)

            for i in xrange(len(currPlotHandles)):
                if currPlotLabels[i] in plotLabels:
                    continue

                if isinstance(currPlotHandles[i], list):
                    plotHandles.append(currPlotHandles[i][0])
                else:
                    plotHandles.append(currPlotHandles[i])

                plotLabels.append(currPlotLabels[i])


        if self.figLegendLoc is not None:
            figLegendKeywords = {}

            if self.figLegendCols is not None:
                versionPieces = [int(x) for x in matplotlib.__version__.split('.')]

                (superMajor, major, minor) = versionPieces[0:3]

                if superMajor == 0 and major < 98:
                    print >>sys.stderr, "Number of columns support not available in versions of matplotlib prior to 0.98"
                else:
                    figLegendKeywords["ncol"] = self.figLegendCols

            pylab.figlegend(plotHandles, plotLabels,
                            self.figLegendLoc,
                            **figLegendKeywords)

        if self.figTitle is not None:
            fig.suptitle(self.figTitle)



import pylab
from matplotlib import pyplot
from PlotInfo import PlotInfo

class Label(PlotInfo):
    def __init__(self, x, y, text=None, bbox=None):
        PlotInfo.__init__(self, "label")
        
        self.x = x
        self.y = y
        self.text = text
        self.textX = x
        self.textY = y
        self.arrow = None
        self.marker = None
        self.rotation = None

        if bbox: self.bbox = dict(bbox)
        else: self.bbox = None

    def setTextOffset(self, x, y):
        self.textX = self.x + x
        self.textY = self.y + y

    def setTextPosition(self, x, y):
        self.textX = x
        self.textY = y

    def hasArrow(self, style="->", color="black"):
        self.arrow = dict(facecolor=color, arrowstyle=style)
    
    def draw(self, axis):
        kwdict = {}
        kwdict["xytext"] = (self.textX, self.textY)
        kwdict["xycoords"] = "data"
        kwdict["textcoords"] = "data"
        kwdict["arrowprops"] = self.arrow
        kwdict["horizontalalignment"] = "center"
        kwdict["rotation"] = self.rotation

        # For props, see
        # http://matplotlib.sourceforge.net/api/artist_api.html#matplotlib.patches.Rectangle
        if self.bbox: kwdict["bbox"] = self.bbox

        handles = []
        labels = []

        handles.append(axis.annotate(self.text, (self.x, self.y), **kwdict))
        labels.append(None)

        if self.marker is not None:
            handles.append(axis.scatter([self.x],[self.y],marker=self.marker, 
                                        color="black"))
            labels.append(None)

        return [handles, labels]



import pylab
import matplotlib
from matplotlib import pyplot
import sys
import os

from Utils import getGoldenRatioDimensions

class PlotLayout(object):
    def __init__(self):
        self.groupedPlots = {}
        self.plots = []
        self.groupOrder = None

        self.width = 1
        self.plotParams = None

        self.dimensions = None
        self.figdimensions = None

        self.figLegendLoc = None
        self.figLegendCols = None

        self.rcParams = None
        
    def __setRCParam(self, param, value):
        if self.rcParams is None:
            self.rcParams = {}

        self.rcParams[param] = value

    def useLatexLabels(self):
        print >>sys.stderr, "WARNING: Using LaTeX labels requires dvipng and ghostscript"
        self.__setRCParam("font.family", "serif")
        self.__setRCParam("font.serif", "Times, Palatino, New Century Schoolbook, Bookman, Computer Modern Roman")
        self.__setRCParam("font.sans-serif", "Helvetica, Avant Garde, Computer Modern Sans serif")
        self.__setRCParam("font.cursive", "Zapf Chancery")
        self.__setRCParam("font.monospace", "Courier, Computer Modern Typewriter")
        self.__setRCParam("text.usetex", True)

    # def useStandardFont(self):
    #     self.__setRCParam("font.family", "serif")
    #     self.__setRCParam("font.style", "normal")
    #     self.__setRCParam("font.variant", "normal")
    #     self.__setRCParam("font.weight", "normal")
    #     self.__setRCParam("font.stretch", "normal")
        
    def setAxesLabelSize(self, size):
        self.__setRCParam("axes.labelsize", size)

    def setXTickLabelSize(self, size):
        self.__setRCParam("xtick.labelsize", size)

    def setYTickLabelSize(self, size):
        self.__setRCParam("ytick.labelsize", size)

    def setLegendLabelSize(self, size):
        self.__setRCParam("legend.fontsize", size)

    def setWidth(self, width):
        self.width = int(width)

    def hasFigLegend(self, loc="best", numcols=1):
        self.figLegendLoc = loc
        self.figLegendCols = numcols

    def addPlot(self, plot, grouping=None):
        if grouping == None:
            self.plots.append(plot)
        else:
            if grouping not in self.groupedPlots:
                self.groupedPlots[grouping] = []

            self.groupedPlots[grouping].append(plot)

    def setGroupOrder(self, groupOrder):
        self.groupOrder = groupOrder

    def setPlotDimensions(self, x, y):
        self.dimensions = (x,y)

    def setFigureDimensions(self, x, y):
        self.figdimensions = (x,y)

    def setPlotParameters(self, **kwdict):
        self.plotParams = dict(kwdict)

        if "left" not in self.plotParams:
            self.plotParams["left"] = 0.12

        if "bottom" not in self.plotParams:
            self.plotParams["bottom"] = 0.10

        if "right" not in self.plotParams:
            self.plotParams["right"] = 0.90

        if "top" not in self.plotParams:
            self.plotParams["top"] = 0.90

        if "wspace" not in self.plotParams:
            self.plotParams["wspace"] = 0.20

        if "hspace" not in self.plotParams:
            self.plotParams["hspace"] = 0.20
            

    def _doPlot(self):
        if len(self.groupedPlots) + len(self.plots) == 0:
            print "PlotLayout.plot(): No data to plot!"
            return

        if self.rcParams is not None:
            pylab.rcParams.update(self.rcParams)

        groupedPlotLengths = [len(plots) for plots in self.groupedPlots.values()]

        if len(groupedPlotLengths) == 0:
            maxRowLength = self.width
        else:
            maxRowLength = max(groupedPlotLengths)
        
        numPlots = len(self.plots)

        if numPlots == 0:
            numExcessRows = 0
        elif numPlots <= maxRowLength:
            numExcessRows = 1
        elif numPlots % maxRowLength == 0:
            numExcessRows = numPlots / maxRowLength
        else:
            numExcessRows = (numPlots / maxRowLength) + 1

        numRows = len(self.groupedPlots.keys()) + numExcessRows

        if self.groupOrder is not None:
            keyList = self.groupOrder
        else:
            keyList = self.groupedPlots.keys()

        currentRow = 0

        if self.figdimensions is not None:
            fig = pyplot.figure(figsize=(self.figdimensions[0], 
                                         self.figdimensions[1]))
        elif self.dimensions is not None:
            fig = pyplot.figure(figsize=(self.dimensions[0] * maxRowLength, 
                                         self.dimensions[1] * numRows))
        else:
            (figWidth, figHeight) = getGoldenRatioDimensions(8.0)
            figWidth *= maxRowLength
            figHeight *= numRows
            fig = pyplot.figure(figsize=(figWidth, figHeight))
            # figWidth = fig.get_figwidth()
            # print figWidth
            # print fig.get_figheight()
            # (goldenWidth, goldenHeight) = getGoldenRatioDimensions(figWidth)
            # fig.set_figheight(goldenHeight)
            # print fig.get_figheight()

        plotHandles = []
        plotLabels = []

        for grouping in keyList:
            currentColumn = 1
            plots = self.groupedPlots[grouping]

            numPlots = len(plots)

            for plot in plots:
                myRows = numRows
                myCols = numPlots
                myPos = (currentRow * numPlots) + currentColumn

                (currPlotHandles, currPlotLabels) = plot.subplot(
                    myRows, myCols, myPos)

                for i in xrange(len(currPlotHandles)):
                    if currPlotLabels[i] in plotLabels:
                        continue

                    if isinstance(currPlotHandles[i], list):
                        plotHandles.append(currPlotHandles[i][0])
                    else:
                        plotHandles.append(currPlotHandles[i])

                    plotLabels.append(currPlotLabels[i])

#                plotHandles.extend(currPlotHandles)
#                plotLabels.extend(currPlotLabels)

                currentColumn += 1
            currentRow += 1

        ungroupedPlotsRemaining = len(self.plots)
        if ungroupedPlotsRemaining > 0:
            currentColumn = 1

            for plot in self.plots:
                if currentColumn == 1:
                    numColumns = min(maxRowLength, ungroupedPlotsRemaining)

                myRows = numRows
                myCols = numColumns
                myPos = (currentRow * numColumns) + currentColumn

                (currPlotHandles, currPlotLabels) = plot.subplot(
                    myRows, myCols, myPos)
                for i in xrange(len(currPlotHandles)):
                    if currPlotLabels[i] in plotLabels:
                        continue

                    if isinstance(currPlotHandles[i], list):
                        plotHandles.append(currPlotHandles[i][0])
                    else:
                        plotHandles.append(currPlotHandles[i])

                    plotLabels.append(currPlotLabels[i])
                
                currentColumn += 1

                if currentColumn > numColumns:
                    currentRow += 1
                    currentColumn = 1

                ungroupedPlotsRemaining -= 1

        if self.figLegendLoc is not None:
            figLegendKeywords = {}

            if self.figLegendCols is not None:
                versionPieces = [int(x) for x in matplotlib.__version__.split('.')]
                
                (superMajor, major, minor) = versionPieces[0:3]
                
                if superMajor == 0 and major < 98:
                    print >>sys.stderr, "Number of columns support not available in versions of matplotlib prior to 0.98"
                else:
                    figLegendKeywords["ncol"] = self.figLegendCols
            
            pylab.figlegend(plotHandles, plotLabels, 
                            self.figLegendLoc, 
                            **figLegendKeywords)

        if self.plotParams is not None:
            pylab.subplots_adjust(left=self.plotParams["left"],
                                  bottom=self.plotParams["bottom"],
                                  right=self.plotParams["right"],
                                  top=self.plotParams["top"],
                                  wspace=self.plotParams["wspace"],
                                  hspace=self.plotParams["hspace"])

    def plot(self):
        self._doPlot()
        if not pylab.isinteractive():
            pylab.show()
        else:
            pylab.draw()

    def save(self, filename, **kwargs):
        print >>sys.stderr, "Saving %s ..." % filename
        
        tempDisplayHack = False
        
        if "DISPLAY" not in os.environ:
            tempDisplayHack = True
            os.environ["DISPLAY"] = ":0.0"
        self._doPlot()
        pylab.savefig(filename,**kwargs)
        pylab.clf()
        
        if tempDisplayHack == True:
            del os.environ["DISPLAY"]



import pylab
from matplotlib import pyplot
from PlotInfo import *

class BoxAndWhisker(PlotInfo):
    """
    Box and whisker plots
    """

    def __init__(self):
        super(BoxAndWhisker,self).__init__("boxplot")
        self.width=None
        self.color="black"
        self.label = None

    def draw(self, axis, transform=None):
        # To be compatible with PlotInfo assumptions
        self.xValues = range(1,len(self.xSequence)+1)
        self.yValues = [0 for x in self.xValues]

        super(BoxAndWhisker,self).draw(axis)

        kwdict = {}

        plotHandles = axis.boxplot(self.xSequence, **kwdict)

        # Picking which part of the plot to use in the legend may
        # require more thought as there are multiple lines in the
        # boxplot, as well as the possibility for outliers.
        # Options are ['medians', 'fliers', 'whiskers', 'boxes', 'caps']
        return [plotHandles['medians'], [self.label]]



import pylab
from matplotlib import pyplot
from PlotInfo import PlotInfo

class VLine(PlotInfo):
    def __init__(self):
        super(VLine,self).__init__("vline")
        
        self.marker = None
        self.markerSize = 8.0
        # TODO Change to width
        self.lineWidth = 1
        self.color = 'black'
        self.lineStyle = '-'
        self.dates = False

    def draw(self, axis):
        # Present to keep the PlotInfo sorting from failing
        self.yValues = [0 for x in self.xValues]

        PlotInfo.draw(self, axis)

        kwdict = {}
        kwdict["linestyle"] = self.lineStyle
        kwdict["color"] = self.color
        kwdict["label"] = self.label
        kwdict["linewidth"] = self.lineWidth

        return [[axis.axvline(x=xValue, **kwdict)
                    for xValue in self.xValues],
                [self.label for xValue in self.xValues]]



#!/usr/bin/env python

from distutils.core import setup

setup(name='boomslang',
      version='0.5.1',
      download_url='http://boomslang.googlecode.com/files/boomslang-0.5.1.tar.gz',
      description='A thin layer over matplotlib that simplifies creation of common plots',
      long_description='''I created Boomslang to decrease the amount of boilerplate
code I had to write when producing graphs for research papers. Boomslang
treats data and plots separately and encapsulates them both in objects,
giving the programmer the ability to author modular, re-usable graphing
code.''',
      author='Alex C. Rasmussen',
      author_email='alexras@acm.org',
      url='http://code.google.com/p/boomslang/',
      keywords=['matplotlib'],
      classifiers = [f.strip() for f in """
                     Development Status :: 4 - Beta
                     Environment :: Other Environment
                     Intended Audience :: Developers
                     License :: OSI Approved :: BSD License
                     Natural Language :: English
                     Operating System :: OS Independent
                     Programming Language :: Python
                     Programming Language :: Python :: 2.5
                     Topic :: Software Development :: Libraries :: Python Modules
                     """.splitlines() if f.strip()],
      license='BSD',
      requires = ['matplotlib'],
      packages=['boomslang'],
      package_dir={'boomslang':''})

# vim: set ts=4 sw=4 expandtab:



import unittest
import os

class TestExamples(unittest.TestCase):
    for example in filter(lambda x: x.endswith('py'), os.listdir('examples')):
        example_no_py = example[:-3]
        full_path = os.path.join('examples', example)
        exec("def test_%s(self):\n" % (example_no_py,) +
             "    res = os.system('python %s')\n" % (full_path,) +
             "    self.assertTrue(res == 0)")

if __name__ == '__main__':
    unittest.main()



import pylab
from matplotlib import pyplot
from matplotlib.transforms import Affine2D, IdentityTransform
from PlotInfo import *
from Bar import *
from StackedBars import *
import sys

class ClusteredBars(PlotInfo):
    """
    A clustered bar chart consisting of multiple series of bars
    with the same X axis values.
    """
    def __init__(self):
        super(ClusteredBars,self).__init__("clustered bar")
        
        self.bars = []
        self.spacing = 0
        self.width = 1
        self.background_color = "white"
        
    def add(self, bar):
        if not isinstance(bar, Bar) and not isinstance(bar, StackedBars):
            print >>sys.stderr, "Can only add Bars to a ClusteredBars"
            sys.exit(1)
        bar.width = self.width
        self.bars.append(bar)
    
    def getXLabelLocations(self):
        labelLocations = []
        clusterWidth = sum([bar.width for bar in self.bars])
        for x in self.bars[0].xValues:
            labelLocations.append((clusterWidth + self.spacing) * float(x) - (self.width / 2.0) + clusterWidth / 2.0)
            
        return labelLocations
    
    def draw(self, axis, transform=None):
        if self.xTickLabels is not None:
            self.xTickLabelPoints = self.getXLabelLocations()
            if len(self.xTickLabelPoints) != len(self.xTickLabels):
                print >>sys.stderr, "Number of clustered bar labels doesn't match number of points"
                print >>sys.stderr, "Labels: %s" % (self.xTickLabels)
                print >>sys.stderr, "Points: %s" % (self.xTickLabelPoints)
                sys.exit(1)
                
        super(ClusteredBars,self).draw(axis)
        
        rect = axis.patch
        rect.set_facecolor(self.background_color)
        
        return self._draw(axis, transform)
        

    def _draw(self, axis, transform=None):
        numBars = len(self.bars)
        
        plotHandles = []
        plotLabels = []
        
        xMin = None
        xMax = None

        clusterWidth = sum([bar.width for bar in self.bars])

        for i in xrange(numBars):
            bar = self.bars[i]

            pre_width = sum([self.bars[j].width for j in xrange(i)])

            bar_xform = Affine2D().scale(clusterWidth + self.spacing, 1)\
                                  .translate(pre_width, 0)

            if transform:
                xform = bar_xform + transform
            else:
                xform = bar_xform

            [handles, labels] = bar._draw(axis, transform=xform)
            bar.drawErrorBars(axis, transform=xform)

            if isinstance(bar, Bar):
                # Only give handle to first rectangle in bar
                plotHandles.append(handles[0])
                plotLabels.append(labels[0])
            else:
                # Assuming StackedBar
                if i == 0:
                    plotHandles.extend(handles)
                    plotLabels.extend(labels)

        scale_xform = Affine2D().scale(clusterWidth + self.spacing, 1)

        xMin = scale_xform.transform((self.bars[0].xValues[0], 0))[0] - \
                    self.bars[0].width / 2.0

        xMax = scale_xform.transform((self.bars[0].xValues[-1], 0))[0] + \
                    clusterWidth - self.bars[0].width / 2.0

        self.xLimits = (xMin, xMax)

        return [plotHandles, plotLabels]
